import * as os from 'os';
import child_process from 'child_process';

import log from 'electron-log';

import { IpcMainApi, PlatformApi } from '@ucap/electron-common';

import { PlatformChannel } from '@ucap/electron-native';

export class AppPlatformService
  implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'appPlatformService';

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @IpcMainApi.Handle(PlatformChannel.networkInfo)
  async handleNetworkInfo(event: Electron.IpcMainEvent): Promise<any> {
    log.debug('handleNetworkInfo');

    const interfaces = os.networkInterfaces();
    const addresses: { ip: string; mac: string }[] = [];

    for (const k in interfaces) {
      for (const k2 in interfaces[k]) {
        const address = interfaces[k][k2];
        if (address.family === 'IPv4' && !address.internal) {
          addresses.push({ ip: address.address, mac: address.mac });
        }
      }
    }

    return addresses;
  }

  @IpcMainApi.Handle(PlatformChannel.execute)
  async handleExecute(
    event: Electron.IpcMainEvent,
    executableName: string
  ): Promise<number> {
    log.debug('handleExecute');

    const executablePath = __WIN32__ ? `${executableName}.exe` : executableName;

    const p = child_process.spawn(executablePath, [], {
      stdio: ['ignore', 'ignore', 'ignore'],
      detached: true
    });

    return p.pid;
  }

  @IpcMainApi.Handle(PlatformChannel.openDefaultBrowser)
  async handleOpenDefaultBrowser(
    event: Electron.IpcMainEvent,
    url: string,
    options?: {
      name?: string;
      features?: string;
      replace?: boolean;
    }
  ): Promise<void> {
    const shell = (window as any).require('electron').shell;
    shell.openExternal(url);
  }

  @IpcMainApi.Handle(PlatformChannel.readFromClipboard)
  async handleReadFromClipboard(
    event: Electron.IpcMainEvent
  ): Promise<{
    text?: string;
    rtf?: string;
    html?: string;
    image?: Buffer;
    imageDataUrl?: string;
  }> {
    log.debug('handleReadFromClipboard');

    const text = Electron.clipboard.readText('clipboard');
    const rtf = Electron.clipboard.readRTF('clipboard');
    const html = Electron.clipboard.readHTML('clipboard');
    const image = Electron.clipboard.readImage('clipboard');

    return {
      text,
      rtf,
      html,
      image: !image.isEmpty() ? image.toBitmap() : undefined,
      imageDataUrl: !image.isEmpty() ? image.toDataURL() : undefined
    };
  }
}
